@@ -34,6 +34,7 @@ config.addV3User(
     snmpEngine, 'test-user',
     config.usmHMACMD5AuthProtocol, 'authkey1',
     config.usmDESPrivProtocol, 'privkey1'
+#    config.usmAesCfb128Protocol, 'privkey1'
     )
 
 # Install default Agent configuration
@@ -44,9 +45,12 @@ config.addV3User(
 
 # Alternatively, configure VACM from the scratch
 config.addContext(snmpEngine, '')
-config.addRwUser(snmpEngine, 1, 'test-agent', 'noAuthNoPriv', (1,3,6,1)) # v1
-config.addRwUser(snmpEngine, 2, 'test-agent', 'noAuthNoPriv', (1,3,6,1)) # v2c
-config.addRwUser(snmpEngine, 3, 'test-user', 'authPriv', (1,3,6,1)) # v3
+config.addVacmUser(snmpEngine, 1, 'test-agent', 'noAuthNoPriv',
+                   (1,3,6), (1,3,6)) # v1
+config.addVacmUser(snmpEngine, 2, 'test-agent', 'noAuthNoPriv',
+                   (1,3,6), (1,3,6)) # v2c
+config.addVacmUser(snmpEngine, 3, 'test-user', 'authPriv',
+                   (1,3,6), (1,3,6)) # v3
 
 # SNMP context
 snmpContext = context.SnmpContext(snmpEngine)
