@@ -2,6 +2,10 @@
 from pysnmp.entity import engine, config
 from pysnmp.carrier.asynsock.dgram import udp
 from pysnmp.entity.rfc3413 import ntfrcv
+from pysnmp import debug
+
+# Optionally enable stdout debugging
+#debug.setLogger(debug.Debug('all'))
 
 # Create SNMP engine with autogenernated engineID and pre-bound
 # to socket transport dispatcher
@@ -22,7 +26,8 @@ config.addV3User(
     snmpEngine, 'test-user',
     config.usmHMACMD5AuthProtocol, 'authkey1',
     config.usmDESPrivProtocol, 'privkey1'
-#    '80004fb81c3dafe69'   # ContextEngineID of Notification Originator
+#   '\x80\x00\x4f\xb8\x1c\x3d\xaf\xe6'   # ContextEngineID of
+                                         # Notification Originator
     )
     
 # Callback function for receiving notifications
