@@ -1,3 +1,4 @@
+import types
 from pysnmp.entity import config
 from pysnmp.entity.rfc3413 import ntforg, context, mibvar
 from pysnmp.entity.rfc3413.oneliner import cmdgen
@@ -9,6 +10,7 @@ usmNoAuthProtocol = cmdgen.usmNoAuthProtocol
 
 # Privacy protocol
 usmDESPrivProtocol = cmdgen.usmDESPrivProtocol
+usmAesCfb128Protocol = cmdgen.usmAesCfb128Protocol
 usmNoPrivProtocol = cmdgen.usmNoPrivProtocol
 
 # Credentials
@@ -24,10 +26,12 @@ class AsynNotificationOriginator(cmdgen.AsynCommandGenerator):
         self.snmpContext = snmpContext
         self.__knownAuths = {}
 
+    def __del__(self): self.uncfgNtfOrg()
+
     def cfgNtfOrg(self, authData, transportTarget, notifyType, tagList=''):
         addrName, paramsName = self.cfgCmdGen(authData, transportTarget,
                                               tagList)
-        notifyName = '%s-name' % tagList
+        notifyName = 'n-%s' % hash(tagList)
         if not self.__knownAuths.has_key(authData):
             config.addNotificationTarget(
                 self.snmpEngine,
@@ -73,12 +77,29 @@ class AsynNotificationOriginator(cmdgen.AsynCommandGenerator):
         tagList = 'notify-list'
         notifyName = self.cfgNtfOrg(authData, transportTarget,
                                     notifyType, tagList)
+        if notificationType:
+            name, oid = mibvar.mibNameToOid(
+                self.mibViewController, notificationType
+                )
+            notificationType = name + oid
         if varBinds:
             __varBinds = []
             for varName, varVal in varBinds:
                 name, oid = mibvar.mibNameToOid(
                     self.mibViewController, varName
                     )
+                if not type(varVal) == types.InstanceType:
+                    ((symName, modName), suffix) = mibvar.oidToMibName(
+                        self.mibViewController, name + oid
+                        )
+                    syntax = mibvar.cloneFromMibValue(
+                        self.mibViewController, modName, symName, varVal
+                        )
+                    if syntax is None:
+                        raise error.PySnmpError(
+                            'Value type MIB lookup failed for %s' % repr(varName)
+                            )
+                    varVal = syntax.clone(varVal)
                 __varBinds.append((name + oid, varVal))
         else:
             __varBinds = None
