@@ -590,6 +590,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
             try:
                 cachedReqParams = self._cachePopByMsgId(msgID)
             except error.ProtocolError:
+                smHandler.releaseStateInformation(securityStateReference)
                 raise error.StatusInformation(
                     errorIndication = 'dataMismatch'
                     )
@@ -617,7 +618,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
             # 7.2.11b (incomplete implementation)
 
             # 7.2.11c
-# XXX            smHandler.releaseStateInformation(securityStateRerefence)
+            smHandler.releaseStateInformation(securityStateReference)
 
             # 7.2.11d
             stateReference = None
@@ -637,11 +638,13 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                securityLevel != cachedReqParams['securityLevel'] or \
                contextEngineId != cachedReqParams['contextEngineId'] or \
                contextName != cachedReqParams['contextName']:
+                smHandler.releaseStateInformation(securityStateReference)
                 raise error.StatusInformation(
                     errorIndication = 'dataMispatch'
                     )
                         
             # 7.2.12c
+            smHandler.releaseStateInformation(securityStateReference)
             stateReference = None
 
             # 7.2.12d
@@ -706,6 +709,8 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
 
         # 7.2.14
         if rfc3411.unconfirmedClassPDUs.has_key(pduType):
+            # This is not specified explicitly in RFC
+            smHandler.releaseStateInformation(securityStateReference)
             return ( messageProcessingModel,
                      securityModel,
                      securityName,
@@ -720,6 +725,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                      statusInformation,
                      None )
 
+        smHandler.releaseStateInformation(securityStateReference)
         raise error.StatusInformation(
             errorIndication = 'unknownPDU'
             )
